@@ -1,25 +1,22 @@
 package Arff::Util;
 
-use warnings;
-use strict;
-
-use Carp qw(cluck);
+use Moose;
 
 use Data::Dumper;
-use Devel::Size qw(size total_size);
 use String::Util ':all';
 
 =head1 NAME
 
 Arff::Util - ARFF files processing utilities.
+This is a moose-based class.
 
 =head1 VERSION
 
-Version 0.01
+Version 1.0
 
 =cut
 
-our $VERSION = '0.01';
+our $VERSION = '1.0';
 
 
 
@@ -42,27 +39,44 @@ our $VERSION = '0.01';
 #		data_record_count => x
 #	      }
 #
-my $relation;
-my $error_count = 0;
+has relation => (
+	is => 'rw',
+	isa => 'HashRef',
+	default => sub{{
+		attributes => [],
+		records => [],
+		data_record_count => 0
+	}}
+);
+# Number of errors occured during the parsing or saving
+has error_count => (
+	is => 'rw'
+);
 
+# set it to one to see detail info
+has debug_mode => (
+	is => 'rw',
+	default => 0
+);
 
 =head1 SYNOPSIS
 
 Quick summary of what the module does.
 
   use ARFF::Util;
-  
+
+  my $arff_object = ARFF::Util->new();
   # load .arff formatted file into the buffer, and return pointer to buffer
-  $arff_hash = load_arff($file_address);
+  $arff_hash = $arff_object->load_arff($file_address);
   
   # save given buffer into the .arff formatted file
-  save_arff($arff_hash, $file_address);
+  $arff_object->save_arff($arff_hash, $file_address);
 
 =head1 DESCRIPTION
 
 ARFF::Util provides a collection of methods for processing ARFF formatted files.
 "An ARFF (Attribute-Relation File Format) file is an ASCII text file that describes a list of instances sharing a set of attributes."
-for more information about ARFF format visit http://weka.sourceforge.net/wekadoc/index.php/en:ARFF_%283.5.1%29
+for more information about ARFF format visit http://www.cs.waikato.ac.nz/~ml/weka/arff.html
 
 =head1 EXPORT
 
@@ -77,15 +91,18 @@ Get arff file path and load it in buffer
 =cut
 
 sub load_arff {
-	my ($arff_file) = @_;
-	local *FILE;
-	open(FILE, $arff_file);
+	my ($self, $arff_file) = @_;
 	my $status = q/normal/;
- 	cluck "Loading $arff_file ...";
+	if($self->debug_mode){
+ 		print "Loading $arff_file ...\n";
+	}
 	my $record_count = 0;
 	my $attribute_count = 0;
 	my $line_counter = 1;
+	my $relation = $self->relation;
 
+	local *FILE;
+	open(FILE, $arff_file)  or die $!;;
 	while (my $current_line = <FILE>)
 	{
 		$current_line = trim($current_line);
@@ -96,12 +113,12 @@ sub load_arff {
 		}
 		elsif ($current_line =~ /^\s*\@RELATION\s+(\S*)/i )
 		{
-			$relation -> {"relation_name"} = $1;
+			$relation->{relation_name} = $1;
 		}
 		elsif ($current_line =~ /^\s*\@ATTRIBUTE\s+(\S*)\s+(\S*)/i )
 		{
-			if(!$relation -> {"attributes"}){
-				$relation -> {"attributes"} = [];
+			if(!$relation->{attributes}){
+				$relation->{attributes} = [];
 			}
 			my $attribute = { "attribute_name" => $1, "attribute_type" => $2};
 			my $attributes = $relation -> {"attributes"};
@@ -137,8 +154,10 @@ sub load_arff {
 				push  @$records , $cur_record;
 			}else
 			{
-				cluck "Line $line_counter : Invalid data record: $current_line Containts ".(scalar @data_parts)." Expected $attribute_count";
-				$error_count++;
+				if($self->debug_mode){
+					print "Line $line_counter : Invalid data record: $current_line Containts ".(scalar @data_parts)." Expected $attribute_count\n";
+				}
+				$self->error_count($self->error_count+1);
 			}
 			
 		}
@@ -147,9 +166,14 @@ sub load_arff {
 	
 	$relation -> {"data_record_count"} = $record_count;
  	$relation -> {"attribute_count"} = $attribute_count;
-# 	cluck Dumper $relation;
-	cluck "$arff_file loaded with $error_count error(s).";
-	cluck "Buffer size: ".total_size($relation)." bytes";
+
+	if($self->debug_mode){
+		eval("use Devel::Size qw(size total_size)");
+
+		print "$arff_file loaded with ".$self->error_count." error(s).\n";
+		print "Buffer size: ".total_size($relation)." bytes\n";
+	}
+	$self->relation($relation);
 	return $relation;
 }
 
@@ -160,32 +184,33 @@ Save given buffer into the .arff formatted file.
 =cut
 
 sub save_arff {
-	my ($buffer,$arff_file) = @_;
+	my ($self, $buffer, $arff_file) = @_;
 	
 	local *FILE;
 	
 	open(FILE, ">$arff_file");
 	
 	my $record_count = 0;
- 	my $error_count = 0;
  	
-	cluck "Writing buffer to $arff_file ...";
+	if($self->debug_mode){
+		print "Writing buffer to $arff_file ...\n";
+	}
 	
-	if($buffer -> {"relation_name"})
+	if($buffer->{relation_name})
 	{
-		print FILE q/@RELATION / . $buffer -> {"relation_name"} . "\n";
+		print FILE q/@RELATION / . $buffer->{relation_name} . "\n";
 	}
 	print FILE "\n";
 	print FILE "\n";
 
-	if($buffer -> {"attributes"}){
+	if($buffer->{attributes}){
 		foreach my $attribute (@{$buffer -> {"attributes"}}){
 			print FILE q/@ATTRIBUTE / . $attribute->{attribute_name} . q/ / . $attribute->{attribute_type} . "\n";
 		}
 		print FILE "\n";
 		print FILE "\n";
 	
-		if($buffer -> {"records"}){
+		if($buffer->{records}){
 			print FILE "\@DATA\n";
 			print FILE "\n";
 
@@ -195,9 +220,11 @@ sub save_arff {
 					if($record->{$attribute->{attribute_name}}){
 						$record_string .= $record->{$attribute->{attribute_name}} . q/,/;
 					}else{
-						cluck "Invali buffer passed, ".$attribute->{attribute_name}." is not defined for record... write UNKNOWN";
+						if($self->debug_mode){
+							print "Invali buffer passed, ".$attribute->{attribute_name}." is not defined for record... write UNKNOWN\n";
+						}
 						$record_string .= q/UNKNOWN,/;
-						$error_count++;
+						$self->error_count($self->error_count+1);
 					}
 				}
 
@@ -209,17 +236,17 @@ sub save_arff {
 		}
 	}
 
-	cluck "Buffer saved to $arff_file with $error_count error(s).";
-	cluck "Buffer size: ".total_size($relation)." bytes";
-	cluck "Data rows count: ".$record_count."";
+	if($self->debug_mode){
+		eval("use Devel::Size qw(size total_size)");
+		
+		print "Buffer saved to $arff_file with ".$self->error_count." error(s).\n";
+		print "Buffer size: ".total_size($buffer)." bytes\n";
+		print "Data rows count: ".$record_count."\n";
+	}
 
 	return 1;
 }
 
-load_arff("c-train.arff");
-
-save_arff($relation , "c-train2.arff");
-
 =head1 AUTHOR
 
 Ehsan Emadzadeh, C<< <ehsan0emadzadeh at yahoo.com> >>
